@@ -38,16 +38,4 @@ testCompressed <- function(x){
   assertCharacter(x = x, any.missing = FALSE, len = 1)
 
   return(grepl("^.*(.gz|.bz2|.tar|.zip|.tgz|.gzip|.7z)[[:space:]]*$", x))
-}
-
-#' View large simple features
-#'
-#' @param x [\code{sf}]\cr the sf object to view.
-#' @importFrom rlang ensym
-
-View_sf <- function(x){
-  theTitle <- ensym(x)
-  x %>% as_tibble() %>%
-    select(-geom) %>%
-    View(title = as.character(theTitle))
 }
\ No newline at end of file
