@@ -17,7 +17,8 @@
 #'   valid.
 #' @param end [\code{integerish(1)}]\cr the date until which the data are valid.
 #' @param schema [\code{list(1)}]\cr the schema description of the table to read
-#'   in (must have been placed in the global environment before calling it here.
+#'   in (must have been placed in the global environment before calling it
+#'   here).
 #' @param archive [\code{character(1)}]\cr the original file from which the
 #'   boundaries emerge.
 #' @param archiveLink [\code{character(1)}]\cr download-link of the archive.
@@ -39,24 +40,23 @@
 #' @param overwrite [\code{logical(1)}]\cr whether or not the geometry to
 #'   register shall overwrite a potentially already existing older version.
 #' @details When processing areal data tables, carry out the following steps:
-#'   \enumerate{ \item Determine the \code{nation}, administrative \code{level},
-#'   a \code{subset} (if applicable) and the \code{dataseries} of the areal data
-#'   and of the geometry, and provide them as arguments to this function. \item
-#'   Provide a \code{begin} and \code{end} date for the areal data. \item Run
-#'   the function. \item (Re)Save the table with the following properties:
-#'   \itemize{\item Format: csv \item Encoding: UTF-8 \item File name: What is
-#'   provided as message by this function \item make sure that the file is not
-#'   modified or reshaped. This will happen during data normalisation via the
-#'   schema description, which expects the original table.} \item Confirm that
-#'   you have saved the file.}
+#'   \enumerate{ \item Determine the \code{nation}, a \code{subset} (if
+#'   applicable), the administrative \code{level} and the \code{dataseries} of
+#'   the areal data and of the geometry, and provide them as arguments to this
+#'   function. \item Provide a \code{begin} and \code{end} date for the areal
+#'   data. \item Run the function. \item (Re)Save the table with the following
+#'   properties: \itemize{\item Format: csv \item Encoding: UTF-8 \item File
+#'   name: What is provided as message by this function \item make sure that the
+#'   file is not modified or reshaped. This will happen during data
+#'   normalisation via the schema description, which expects the original
+#'   table.} \item Confirm that you have saved the file.}
 #'
 #'   Every areal data dataseries (\code{dSeries}) may come as a slight
 #'   permutation of a particular table arrangement. The function
 #'   \code{\link{normTable}} expects internally a schema description (a list
 #'   that describes the position of the data components) for each data table,
-#'   which is saved as \code{paste0("meta_", dSeries, TAB_NUMBER)}. A template
-#'   thereof, and documentation on how to set them up, can be found in
-#'   \code{tabshiftr::\link{makeSchema}} with \code{arealDB}.
+#'   which is saved as \code{paste0("meta_", dSeries, TAB_NUMBER)}. See package
+#'   \code{tabshiftr}.
 #' @return Returns a tibble of the entry that is appended to 'inv_tables.csv' in
 #'   case \code{update = TRUE}.
 #' @examples
@@ -65,24 +65,19 @@
 #'
 #' # the schema description for this table
 #' library(tabshiftr)
-#' schema_madeUp <- makeSchema(
-#'   list(header = list(row = 1),
-#'        variables = list(
-#'          al1 =
-#'            list(type = "id", col = 1),
-#'          year =
-#'            list(type = "id", col = 2),
-#'          commodities =
-#'            list(type = "id", col = 3),
-#'          harvested =
-#'            list(type = "measured", unit = "ha",
-#'                 factor = 1, col = 4),
-#'          production =
-#'            list(type = "measured", unit = "t",
-#'                 factor = 1, col = 5))))
+#' library(magrittr)
 #'
-#' regTable(nation = "estonia",
-#'          subset = "soyMaize",
+#' schema_madeUp <-
+#'   setIDVar(name = "al1", columns = 1) %>%
+#'   setIDVar(name = "year", columns = 2) %>%
+#'   setIDVar(name = "commodities", columns = 3) %>%
+#'   setObsVar(name = "harvested",
+#'             factor = 1, columns = 4) %>%
+#'   setObsVar(name = "production",
+#'             factor = 1, columns = 5)
+#'
+#' regTable(nation = "Estonia",
+#'          subset = "barleyMaize",
 #'          dSeries = "madeUp",
 #'          gSeries = "gadm",
 #'          level = 1,
@@ -97,9 +92,10 @@
 #'          metadataPath = "my/local/path",
 #'          update = TRUE)
 #' @importFrom readr read_csv write_rds guess_encoding
+#' @importFrom rlang ensym
 #' @importFrom checkmate assertDataFrame assertNames assertCharacter
 #'   assertIntegerish assertSubset assertLogical testChoice assertChoice
-#'   assertFileExists assertClass assertTRUE
+#'   assertFileExists assertClass assertTRUE testDataFrame testNames
 #' @importFrom dplyr filter distinct
 #' @importFrom stringr str_split
 #' @importFrom tibble tibble
@@ -159,12 +155,17 @@ regTable <- function(nation = NULL, subset = NULL, dSeries = NULL, gSeries = NUL
   assertCharacter(x = metadataPath, any.missing = FALSE, null.ok = TRUE)
   assertCharacter(x = notes, ignore.case = TRUE, any.missing = FALSE, len = 1, null.ok = TRUE)
   assertLogical(x = update, len = 1)
+  assertLogical(x = overwrite, len = 1)
+
+  schemaName <- as.character(substitute(schema))
 
   # ask for missing and required arguments
   if(!is.null(subset)){
     if(grepl(pattern = "_", x = subset)){
       stop("please give a subset that does not contain any '_' characters.")
     }
+  } else {
+    subset <- ""
   }
 
   if(is.null(dSeries)){
@@ -172,7 +173,7 @@ regTable <- function(nation = NULL, subset = NULL, dSeries = NULL, gSeries = NUL
     if(!testing){
       dSeries <- readline()
     } else {
-      dSeries <- "maia"
+      dSeries <- "madeUp"
     }
 
     if(grepl(pattern = "_", x = dSeries)){
@@ -251,7 +252,7 @@ regTable <- function(nation = NULL, subset = NULL, dSeries = NULL, gSeries = NUL
     if(!testing){
       end <- readline()
     } else {
-      end <- as.integer(format(Sys.Date(), "%Y"))
+      end <- 2017
     }
     if(is.na(end)){
       end =  NA_integer_
@@ -263,13 +264,14 @@ regTable <- function(nation = NULL, subset = NULL, dSeries = NULL, gSeries = NUL
     if(!testing){
       schema <- readline()
     } else {
-      schema <- tabshiftr::schema_default
+      schema <- readRDS(file = paste0(intPaths, "/adb_tables/meta/schemas/example_schema.rds"))
     }
     if(length(schema) < 1){
       schema = NA_character_
     }
   }
 
+
   if(is.null(archive)){
     message("please type in the archives' file name: ")
     if(!testing){
@@ -283,10 +285,10 @@ regTable <- function(nation = NULL, subset = NULL, dSeries = NULL, gSeries = NUL
   }
 
   # determine nation value
-  if(!testChoice(x = tolower(nation), choices = countries$nation)){
+  if(!testChoice(x = nation, choices = countries$nation)){
     theNation <- NULL
   } else{
-    nations <- tolower(nation)
+    nations <- nation
     assertChoice(x = nations, choices = countries$nation)
     theNation <- countries %>%
       as_tibble() %>%
@@ -303,6 +305,7 @@ regTable <- function(nation = NULL, subset = NULL, dSeries = NULL, gSeries = NUL
   if(any(inv_tables$source_file %in% fileName)){
     if(overwrite){
       theSchemaName <- inv_tables$schema[inv_tables$source_file == fileName]
+      newTID <- inv_tables$tabID[which(inv_tables$source_file %in% fileName)]
     } else {
       return(paste0("'", fileName, "' has already been registered."))
     }
@@ -311,7 +314,7 @@ regTable <- function(nation = NULL, subset = NULL, dSeries = NULL, gSeries = NUL
   }
 
   # make a schema description
-  write_rds(x = schema, path = paste0(intPaths, "/adb_tables/meta/schemas/", theSchemaName, ".rds"))
+  write_rds(x = schema, file = paste0(intPaths, "/adb_tables/meta/schemas/", theSchemaName, ".rds"))
 
   if(is.null(archiveLink)){
     message("please type in the weblink from which the archive was downloaded: ")
@@ -394,37 +397,45 @@ regTable <- function(nation = NULL, subset = NULL, dSeries = NULL, gSeries = NUL
     notes = NA_character_
   }
 
-  # test whether the archive file is available
-  if(!testFileExists(x = paste0(intPaths, "/adb_tables/stage1/", fileArchive[1]), "r")){
-    message(paste0("... please store the archive '", fileArchive[[1]], "' in './adb_tables/stage1'"))
-    if(!testing){
-      done <- readline(" -> press any key when done: ")
-    }
-
-    # make sure that the file is really there
-    assertFileExists(x = paste0(intPaths, "/adb_tables/stage1/", fileArchive[1]), "r")
-
-    # ... and if it is compressed, whether also the file therein is given that contains the data
-    if(testCompressed(x = fileArchive[1]) & length(fileArchive) < 2){
-      message(paste0("please give the name of the file in ", fileArchive[1]," that contains the table: "))
-      if(!testing){
-        theArchiveFile <- readline()
-      } else {
-        theArchiveFile <- "example_table.csv"
-      }
-      archive <- paste0(archive, "|", theArchiveFile)
-    }
-  }
-
   if(update){
+
+    # test whether the archive file is available
+    if(!testFileExists(x = paste0(intPaths, "/adb_tables/stage1/", fileArchive[1]))){
+      message(paste0("... please store the archive '", fileArchive[[1]], "' in './adb_tables/stage1'"))
+      if(!testing){
+        done <- readline(" -> press any key when done: ")
+      }
+
+      # make sure that the file is really there
+      assertFileExists(x = paste0(intPaths, "/adb_tables/stage1/", fileArchive[1]))
+
+      # ... and if it is compressed, whether also the file therein is given that contains the data
+      if(testCompressed(x = fileArchive[1]) & length(fileArchive) < 2){
+        message(paste0("please give the name of the file in ", fileArchive[1]," that contains the table: "))
+        if(!testing){
+          theArchiveFile <- readline()
+        } else {
+          theArchiveFile <- "example_table.csv"
+        }
+        archive <- paste0(archive, "|", theArchiveFile)
+      }
+    }
+
     # test that the file is available
-    if(!testFileExists(x = filePath, "r", extension = "csv")){
+    if(!testFileExists(x = filePath, extension = "csv")){
+      processedPath <- paste0(intPaths, "/adb_tables/stage2/processed/", fileName)
+      if(testFileExists(x = processedPath, extension = "csv")){
+        temp <- inv_tables[which(inv_tables$source_file %in% fileName), ]
+        message(paste0("! the table '", fileName, "' has already been normalised !"))
+        return(temp)
+      }
+
       message(paste0("... please store the table as '", fileName, "' with utf-8 encoding in './adb_tables/stage2'"))
       if(!testing){
         done <- readline(" -> press any key when done: ")
       }
       # make sure that the file is really there
-      assertFileExists(x = filePath, "r", extension = "csv")
+      assertFileExists(x = filePath, extension = "csv")
     }
 
     # put together new census database entry
@@ -441,13 +452,44 @@ regTable <- function(nation = NULL, subset = NULL, dSeries = NULL, gSeries = NUL
                   metadata_link = metadataLink,
                   metadata_path = metadataPath,
                   notes = notes)
-    if(!any(inv_tables$source_file %in% fileName)){
+
+    if(!any(inv_tables$source_file %in% fileName) | overwrite){
       # in case the user wants to update, attach the new information to the table inv_sourceData.csv
-      updateTable(index = doc, name = "inv_tables")
+      updateTable(index = doc, name = "inv_tables", matchCols = c("source_file"))
     }
+
     return(doc)
   } else {
-    message(paste0("... the filename is '", fileName, "'."))
+
+    stage1Exists <- testFileExists(x = filePath, "r", extension = "csv")
+    stage2Exists <- testFileExists(x = paste0(intPaths, "/adb_tables/stage1/", fileArchive[1]), "r")
+    if(stage1Exists){
+      thisTable <- as_tibble(read.csv(file = filePath, header = FALSE, as.is = TRUE, na.strings = schema@format$na, encoding = "UTF-8"))
+      temp <- tryCatch(expr = reorganise(input = thisTable, schema = schema),
+                       error = function(e){
+                         return("There was an error message")
+                       },
+                       warning = function(w){
+                         return("There was a warning message")
+                       })
+      isTable <- testDataFrame(x = temp)
+      correctNames <- testNames(x = names(temp), must.include = names(schema@variables))
+      schema_ok <- if_else(isTable & correctNames, "yes", temp)
+    } else {
+      schema_ok <- ""
+    }
+
+    diag <- tibble(stage1_name = fileArchive[1],
+                   stage2_name = fileName,
+                   schema_name = schemaName,
+                   stage1_ok = stage2Exists,
+                   stage2_ok = stage1Exists,
+                   schema_ok = schema_ok)
+
+    updateTable(index = diag, name = "diag_tables", matchCols = c("stage2_name"), backup = FALSE)
+
+    return(diag)
   }
 
+
 }
